import { Component, Input, OnInit, OnChanges, TemplateRef, SimpleChanges } from '@angular/core';
import { merge, Observable, timer } from 'rxjs';
import { switchMap, debounce, distinctUntilChanged, tap, partition } from 'rxjs/operators';
import { NgxHttpLoaderInterceptor } from './ngx-loader-interceptor.service';
import { NgxLoader } from './ngx-http-loader';
import { NgxHttpLoaderService } from './ngx-http-loader.service';
import { DomSanitizer } from '@angular/platform-browser';

@Component({
  selector: 'ngx-http-loader',
  templateUrl: './ngx-http-loader.component.html',
  styleUrls: ['./ngx-http-loader.component.scss']
})
export class NgxHttpLoaderComponent implements OnInit {

  public customEnumLoader = NgxLoader;
  @Input() public customTemplate!: TemplateRef<any>;
  @Input() public customLoader!: TemplateRef<any>;
  @Input() public loader: string | null = NgxLoader.RIPPLE;
  @Input() public filterUrlRegexPatterns: RegExp[] = [];
  @Input() public filterUrls: string[] = [];
  @Input() public filterHttpHeaders: string[] = [];
  @Input() public filterHttpMethods: string[] = [];
  @Input() public delay = 0;
  @Input() public extraDelay = 0;
  @Input() public minDuration = 0;
  @Input() public opacity: number = 0.8;
  // @Input() public backdropBackgroundColor = 'linear-gradient(45deg, #222428, #444856)';
  // @Input() public backdropBackgroundColor = 'transperent';
  @Input() public backdropBackgroundColor = 'linear-gradient(0deg, black, transparent);';
  @Input() public iconColor: string = 'white';
  @Input() public title: string = '';
  @Input() public titleColor: string = '#fff';
  @Input() public titleSize: string = '3rem';
  @Input() public subTitle: string = '';
  @Input() public subTitleColor: string = '#6c757d';
  @Input() public subTitleSize: string = '1.2rem';
  @Input() public custom: boolean = false;

  public show$!: Observable<boolean>;
  private showuntil = Date.now();

  constructor(
    private requestService: NgxHttpLoaderInterceptor,
    public service: NgxHttpLoaderService,
    public sanitizer: DomSanitizer
  ) {
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (this.filterUrlRegexPatterns.length > 0 || this.filterUrls.length > 0
      || this.filterHttpHeaders.length > 0 || this.filterHttpMethods.length > 0) this.initAllFilters();
  }

  ngOnInit(): void {
    this.initAllFilters();
  }

  initAllFilters() {
    this.requestService.setUrlPatterns = this.filterUrlRegexPatterns;
    this.requestService.setFilterUrls = this.filterUrls;
    this.requestService.setFilterMethods = this.filterHttpMethods;
    this.requestService.setFilterHeaders = this.filterHttpHeaders;
    this.initShowObservable();
  }

  initShowObservable() {
    const [show, hide] = partition(($hide: boolean) => $hide)(this.requestService.reqStatus);
    this.show$ = merge(
      this.requestService.reqStatus.pipe(switchMap(() => show.pipe(debounce(() => timer(this.delay))))),
      show.pipe(switchMap(() => hide.pipe(debounce(() => this.showUpto())))),
      this.service.isShowLoader
    ).pipe(distinctUntilChanged(), tap(x => this.updateHideDelay(x)))
  }

  private updateHideDelay(bool: boolean): void {
    if (bool) this.showuntil = Date.now() + this.minDuration;
  }

  private showUpto(): Observable<number> {
    return timer(Math.max(this.extraDelay, this.showuntil - Date.now()));
  }

}
