import { HTTP_INTERCEPTORS, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { ExistingProvider, Injectable } from '@angular/core';
import { Observable, ReplaySubject } from 'rxjs';
import { finalize } from 'rxjs/operators';

@Injectable({
    providedIn: 'root'
})
export class NgxHttpLoaderInterceptor implements HttpInterceptor {

    private reqStatus$ = new ReplaySubject<boolean>(1);
    private pendingReqCnt: number = 0;
    private forceStop: boolean = false;
    private filterUrls: string[] = [];
    private filterUrlPatterns: RegExp[] = [];
    private filterMethods: string[] = [];
    private filterHeaders: string[] = [];

    get pendingRequestCnt(): number {
        return this.pendingRequestCnt;
    }

    get reqStatus(): Observable<boolean> {
        return this.reqStatus$.asObservable();
    }

    set setFilterUrls(urls: string[]) {
        this.filterUrls = urls;
    }

    set setUrlPatterns(regx: RegExp[]) {
        this.filterUrlPatterns = regx;
    }

    set setFilterMethods(methods: string[]) {
        this.filterMethods = methods;
    }

    set setFilterHeaders(headers: string[]) {
        this.filterHeaders = headers;
    }

    set setForceStopReq(bool: boolean) {
        this.forceStop = bool;
    }

    private checkIsFilterUrl(url: string): boolean {
        return this.filterUrls.some(x => x === url)
    }

    private checkUrlPatterns(url: string): boolean {
        return this.filterUrlPatterns.some(x => x.test(url));
    }

    private checkHttpMethod(req: HttpRequest<any>): boolean {
        return this.filterMethods.some(e => {
            return e.toLowerCase() === req.method.toLowerCase();
        });
    }

    private checkReqHeader(req: HttpRequest<any>): boolean {
        return this.filterHeaders.some(e => {
            return req.headers.has(e);
        });
    }

    private isStopReq(r: HttpRequest<any>): boolean {
        return this.forceStop
            || this.checkIsFilterUrl(r.urlWithParams)
            || this.checkUrlPatterns(r.urlWithParams)
            || this.checkHttpMethod(r)
            || this.checkReqHeader(r)
    }

    intercept(
        request: HttpRequest<any>,
        next: HttpHandler,
    ): Observable<HttpEvent<any>> {
        const isNext = this.isStopReq(request);
        if (!isNext) {
            this.pendingReqCnt++;

            if (1 === this.pendingReqCnt) {
                this.reqStatus$.next(true);
            }
        }
        return next.handle(request).pipe(finalize(() => {
            if (!isNext) {
                this.pendingReqCnt--;
                if (0 === this.pendingReqCnt) {
                    this.reqStatus$.next(false);
                }
            }
        }));
    }


}

export const RequestsInterceptorProvider: ExistingProvider[] = [{
    provide: HTTP_INTERCEPTORS,
    useExisting: NgxHttpLoaderInterceptor,
    multi: true
}];
